@@ -76,41 +76,39 @@ class Launcher(BaseLauncher):
         self.proc_def = proc_def
         self.args = args
 
-    async def run(self):
-        life_cycle.set_state(self.name, life_cycle.State.STARTING)
-
+    async def activate_conda_env(self) -> dict:
         # We grab the conda env variables separate from executing the run
         # command to simplify detecting pid and removing some race conditions.
-
+        subprocess_env = os.environ.copy()
+        subprocess_env.update(self.proc_def.env)
         envinfo = await asyncio.create_subprocess_shell(
             f"""
                 eval "$(conda shell.bash hook)"
                 conda activate fbrp_{self.name}
-                cp /proc/self/environ /tmp/fbrp_conda_{self.name}.env
+                cp -f /proc/self/environ /tmp/fbrp_conda_{self.name}.env
             """,
             stdout=asyncio.subprocess.PIPE,
             executable="/bin/bash",
             cwd=self.proc_def.root,
+            env=subprocess_env,
         )
 
         await envinfo.wait()
         lines = open(f"/tmp/fbrp_conda_{self.name}.env").read().split("\0")
         conda_env = dict(line.split("=", 1) for line in lines if "=" in line)
+        return conda_env
 
-        cmd = self.run_command
-        if type(self.run_command) == list:
-            cmd = shlex.join(self.run_command)
-
+    async def run_cmd_in_env(self, conda_env):
         self.proc = await asyncio.create_subprocess_shell(
-            cmd,
+            util.shell_join(self.run_command),
             stdout=asyncio.subprocess.PIPE,
             stderr=asyncio.subprocess.PIPE,
             executable="/bin/bash",
             cwd=self.proc_def.root,
             env=conda_env,
         )
-        life_cycle.set_state(self.name, life_cycle.State.STARTED)
 
+    async def gather_cmd_outputs(self):
         async def log_pipe(logger, pipe):
             while True:
                 line = await pipe.readline()
@@ -127,14 +125,23 @@ class Launcher(BaseLauncher):
             self.down_watcher(self.handle_down),
         )
 
+    async def run(self):
+        life_cycle.set_state(self.name, life_cycle.State.STARTING)
+        conda_env = await self.activate_conda_env()
+        await self.run_cmd_in_env(conda_env)
+        life_cycle.set_state(self.name, life_cycle.State.STARTED)
+        await self.gather_cmd_outputs()
+
     def get_pid(self):
         return self.proc.pid
 
-    async def death_handler(self):
+    async def exit_cmd_in_env(self) -> int:
         await self.proc.wait()
-        life_cycle.set_state(
-            self.name, life_cycle.State.STOPPED, return_code=self.proc.returncode
-        )
+        return self.proc.returncode
+
+    async def death_handler(self):
+        ret_code = await self.exit_cmd_in_env()
+        life_cycle.set_state(self.name, life_cycle.State.STOPPED, return_code=ret_code)
         # TODO(lshamis): Restart policy goes here.
 
     async def handle_down(self):
@@ -206,8 +213,14 @@ class Conda(BaseRuntime):
         print(f"creating conda env for {name}. This will take a minute...")
 
         update_bin = "mamba" if self.use_mamba else "conda"
+        # https://github.com/conda/conda/issues/7279
+        # Updating an existing environment does not remove old packages, even with --prune.
+        subprocess.run(
+            [update_bin, "env", "remove", "-n", env_name],
+            capture_output=not args.verbose,
+        )
         result = subprocess.run(
-            [update_bin, "env", "update", "-f", env_path],
+            [update_bin, "env", "update", "--prune", "-f", env_path],
             capture_output=not args.verbose,
         )
         if result.returncode:
@@ -215,7 +228,9 @@ class Conda(BaseRuntime):
 
         if self.setup_commands:
             print(f"setting up conda env for {name}")
-            setup_command = "\n".join([shlex.join(cmd) for cmd in self.setup_commands])
+            setup_command = "\n".join(
+                [util.shell_join(cmd) for cmd in self.setup_commands]
+            )
             result = subprocess.run(
                 f"""
                     eval "$(conda shell.bash hook)"
